import 'package:cramr_ui_lib/ads/safe_banner_ad_widget.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_evaluate_route.dart';
import 'package:cramr_ui_lib/widgets/delayed_widget_builder.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import '../model/verb_test_model.dart';
import '../settings/settings_controller.dart';
import '../theme/custom_theme.dart';
import '../widgets/streak_counter_widget.dart';

class StreakCongratulationRoute extends StatefulWidget {
  static const routeName = '/streakCongratulationRoute';

  final SettingsController _settingsController;
  final SafeBannerAdWidget safeBannerAdWidget;

  const StreakCongratulationRoute(this._settingsController,
      {Key? key, required this.safeBannerAdWidget})
      : super(key: key);

  static Key toEvaluationButtonKey(String infinitive) {
    return Key('evaluate_$infinitive');
  }

  @override
  State<StreakCongratulationRoute> createState() =>
      StreakCongratulationRouteState();
}

class StreakCongratulationRouteState extends State<StreakCongratulationRoute> {
  @override
  Widget build(BuildContext context) {
    final int streakCount =
        widget._settingsController.streakEngine.streakCount();
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return AnimatedBuilder(
        animation: widget._settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
            debugShowCheckedModeBanner: false,
            themeMode: widget._settingsController.themeMode,
            theme: CustomTheme.lightTheme,
            darkTheme: ThemeData.dark(),
            home: SafeArea(
                child: Scaffold(
              body: Column(
                children: [
                  SingleChildScrollView(
                    child: SizedBox(
                        height: MediaQuery.of(context).size.height * 0.5,
                        child: DelayedWidgetBuilder(
                            delay: const Duration(milliseconds: 500),
                            child: StreakCounterWidget(streakCount))),
                  ),
                  Expanded(child: widget.safeBannerAdWidget)
                ],
              ),
              floatingActionButton: FloatingActionButton(
                key: StreakCongratulationRoute.toEvaluationButtonKey(
                    testModel.infinitive),
                onPressed: () {
                  Navigator.pushNamed(
                      context, VerbIndicativeEvaluateRoute.routeName);
                },
                child: const Icon(Icons.arrow_forward),
              ),
            )),
          );
        });
  }
}
