import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_form_appbar.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_test_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:cramr_ui_lib/widgets/written_indicative_study_widget.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../widgets/conjugation_meta_data_card.dart';

/// The screen for studying in write mode.
class VerbIndicativeStudyWriteModeRoute extends StatelessWidget {
  static const routeName = '/verbIndicativeWriteModeStudyRoute';

  final SettingsController _settingsController;

  const VerbIndicativeStudyWriteModeRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: _settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
            title: "Verbomatique",
            debugShowCheckedModeBanner: false,
            themeMode: _settingsController.themeMode,
            theme: CustomTheme.lightTheme,
            darkTheme: ThemeData.dark(),
            home: SafeArea(
              key: UniqueKey(),
              child: Scaffold(
                key: UniqueKey(),
                appBar: VerbIndicativeStudyFormAppbar().asWidget(context),
                body: ListView(
                  children: [
                    _conjugationMetaDataCard(context),
                    const Divider(),
                    _writtenIndicativeStudyWidget(context),
                  ],
                ),
                floatingActionButton: _toTestRouteButton(context),
              ),
            ));
      },
    );
  }

  Widget _writtenIndicativeStudyWidget(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return WrittenIndicativeStudyWidget(
        targetLanguage: testModel.targetLanguageIso,
        expectedIndicative: testModel.expected,
        numberOfItems: testModel.items.length,
        fullPronouns: testModel.pronouns,
      );
    });
  }

  Widget _conjugationMetaDataCard(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return ConjugationMetaDataCard(
          infinitive: testModel.infinitive,
          translation: testModel.translation,
          mode: testModel.mode,
          tense: testModel.tense);
    });
  }

  Widget _toTestRouteButton(BuildContext context) {
    return FloatingActionButton(
      key: const Key('write_forward_button'),
      child: const Icon(Icons.arrow_forward),
      onPressed: () {
        final StudyModeState studyModeState =
            Provider.of<StudyModeState>(context, listen: false);
        studyModeState.updateState(StudyMode.writtenMode);
        Navigator.pushNamed(context, VerbIndicativeTestRoute.routeName);
      },
    );
  }
}
