import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/material.dart';

/// This widget shows a pronoun and an indicative, based on the person that is
/// passed as a parameter.
class ExpectedRow extends StatelessWidget {
  final int currentPerson;
  final String fullPronoun;
  final String expectedIndicative;
  final String targetLanguage;

  const ExpectedRow(
      {Key? key,
      required this.currentPerson,
      required this.fullPronoun,
      required this.expectedIndicative,
      required this.targetLanguage})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 20,
      child: Row(
        children: [
          PronounField(
            fullPronoun: fullPronoun,
            expectedIndicative: expectedIndicative,
            targetLanguage: targetLanguage,
          ),
          Expanded(child: _expected(context)),
        ],
      ),
    );
  }

  Widget _expected(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(5.0),
      child: SizedBox(
          height: 40,
          child: Text(
            expectedIndicative,
            key: const Key('expectedVerb'),
            style: const TextStyle(
              color: Colors.green,
              fontSize: 25,
            ),
            overflow: TextOverflow.ellipsis,
            textScaleFactor: TextSizeLimiter.textScaleFactor(context),
          )),
    );
  }
}
