import 'package:cramr_ui_lib/model/conjugation.dart';
import 'package:cramr_ui_lib/model/french_verb.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  var infinitive = 'infinitief';
  var translation = 'translation';
  var simplePresentIndicative = ['nul', 'een', 'twee', 'drie', 'vier', 'vijf'];
  var simplePastIndicative = [
    'oudnul',
    'oudeen',
    'oudtwee',
    'ouddrie',
    'oudvier',
    'oudvijf'
  ];
  var isoLanguage = 'fr';

  group('FrenchVerb tests simple present indicative', () {
    test('FrenchVerb state tests', () {
      Conjugation conjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      var model = FrenchVerb(infinitive, translation, conjugation);
      expect(infinitive, model.infinitive);
      expect(translation, model.translation);
      expect(isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in simplePresentIndicative) {
        expect(element, model.simplePresentIndicative[i++]);
      }
      if (null != model.simplePastIndicative) {
        fail('non null where null expected');
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation conjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      var model = FrenchVerb(infinitive, translation, conjugation);
      expect(false, model.hasSimplePastIndicative());
    });
  });

  group('FrenchVerb tests several indicatives', () {
    test('FrenchVerb state tests', () {
      Conjugation presentConjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      Conjugation pastConjugation =
          Conjugation('indicatif', 'imparfait', simplePastIndicative);
      var model = FrenchVerb.fromMoreTensesAndModes(
          infinitive, translation, presentConjugation, pastConjugation);
      expect(infinitive, model.infinitive);
      expect(translation, model.translation);
      expect(isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in simplePastIndicative) {
        expect(element, model.simplePastIndicative![i++]);
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation presentConjugation =
          Conjugation('indicatif', 'présent', simplePresentIndicative);
      Conjugation pastConjugation =
          Conjugation('indicatif', 'imparfait', simplePastIndicative);
      var model = FrenchVerb.fromMoreTensesAndModes(
          infinitive, translation, presentConjugation, pastConjugation);
      expect(true, model.hasSimplePastIndicative());
    });
  });
}
