//TODO: introduce mocking
import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_cover_mode_route.dart';
import 'package:cramr_ui_lib/settings/notification_service.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

final List<String> _randomIndicative = [
  'exp1',
  'exp2',
  'exp3',
  'exp4',
  'exp5',
  'exp6',
];

final List<String> _randomPronouns = [
  'pron1',
  'pron2',
  'pron3',
  'pron4',
  'pron5',
  'pron6',
];

const String _randomInfinitive = 'infinitive';
const String _mode = 'mode';
const String _tense = 'tense';

void main() {
  testWidgets('Testing for the presence of covered buttons',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(_randomIndicative, _randomPronouns,
                _randomInfinitive, 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.flashCardMode);
          })
        ],
        child: VerbIndicativeStudyCoverModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs,
              NotificationService()),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    for (int i = 0; i < _randomIndicative.length; i++) {
      // Make sure that we cannot read the indicative yet
      final Finder indicativeFinder = find.text(_randomIndicative[i]);
      expect(indicativeFinder, findsNothing);

      // Check if we can find the cover
      final Finder coverFinder =
          find.byKey(Key('coverkey_${_randomInfinitive}_$i'));
      await tester.scrollUntilVisible(coverFinder, 10.0);
      expect(coverFinder, findsOneWidget);

      // Make sure that we can read the indicative after tapping it
      await tester.tap(coverFinder);
      await tester.pump();
      expect(indicativeFinder, findsOneWidget);
    }
  });

  testWidgets('Testing for the presence of covered buttons singular only',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(true, false);
    var filteredIndicative =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    var filteredPronouns =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicative, filteredPronouns,
                _randomInfinitive, 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.flashCardMode);
          })
        ],
        child: VerbIndicativeStudyCoverModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs,
              NotificationService()),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    for (int i = 0; i < filteredIndicative.length; i++) {
      // Make sure that we cannot read the indicative yet
      final Finder indicativeFinder = find.text(filteredIndicative[i]);
      expect(indicativeFinder, findsNothing);

      // Check if we can find the cover
      final Finder coverFinder =
          find.byKey(Key('coverkey_${_randomInfinitive}_$i'));
      expect(coverFinder, findsOneWidget);

      // Make sure that we can read the indicative after tapping it
      await tester.tap(coverFinder);
      await tester.pump();
      expect(indicativeFinder, findsOneWidget);
    }
    // Assert that we are unable to find a superfluous cover
    final Finder superfluousCoverFinder =
        find.byKey(const Key('coverkey_${_randomInfinitive}_3'));
    expect(superfluousCoverFinder, findsNothing);
  });

  testWidgets('Testing for the presence of covered buttons plural only',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(false, true);
    var filteredIndicative =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    var filteredPronouns =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicative, filteredPronouns,
                _randomInfinitive, 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.flashCardMode);
          })
        ],
        child: VerbIndicativeStudyCoverModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs,
              NotificationService()),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    for (int i = 0; i < filteredIndicative.length; i++) {
      // Make sure that we cannot read the indicative yet
      final Finder indicativeFinder = find.text(filteredIndicative[i]);
      expect(indicativeFinder, findsNothing);

      // Check if we can find the cover
      final Finder coverFinder =
          find.byKey(Key('coverkey_${_randomInfinitive}_$i'));
      expect(coverFinder, findsOneWidget);

      // Make sure that we can read the indicative after tapping it
      await tester.tap(coverFinder);
      await tester.pump();
      expect(indicativeFinder, findsOneWidget);
    }
    // Assert that we are unable to find a superfluous cover
    final Finder superfluousCoverFinder =
        find.byKey(const Key('coverkey_${_randomInfinitive}_3'));
    expect(superfluousCoverFinder, findsNothing);
  });
}
