import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_write_mode_route.dart';
import 'package:cramr_ui_lib/settings/notification_service.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

final List<String> _randomIndicative = [
  'aime',
  'aimes',
  'aime',
  'aimons',
  'aimez',
  'aiment',
];

final List<String> _randomPronouns = [
  'je',
  'tu',
  'il/elle',
  'nous',
  'vous',
  'ils/elles',
];

final Finder nextPersonKeyFinder = find.byKey(const Key('nextPersonKey'));
final Finder expectedVerbFinder = find.byKey(const Key('expectedVerb'));
final List<Finder> actualVerbFinder = [
  find.byKey(const Key('actualVerb_0')),
  find.byKey(const Key('actualVerb_1')),
  find.byKey(const Key('actualVerb_2')),
  find.byKey(const Key('actualVerb_3')),
  find.byKey(const Key('actualVerb_4')),
  find.byKey(const Key('actualVerb_5')),
];
final Finder statusIconFinder = find.byKey(const Key('comparisonStatusIcon'));
final Finder iconFinder = find.byIcon(Icons.assignment_turned_in_outlined);

const String _mode = 'mode';
const String _tense = 'tense';

void main() {
  testWidgets('Test the state of the verb', (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(_randomIndicative, _randomPronouns, 'infinitive',
                'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.writtenMode);
          })
        ],
        child: VerbIndicativeStudyWriteModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs,
              NotificationService()),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    expect(expectedVerbFinder, findsOneWidget);
    Finder expectedVerbValueFinder = find.text('aime');
    expect(expectedVerbValueFinder, findsOneWidget);
    expect(actualVerbFinder[0], findsOneWidget);
    expect(statusIconFinder, findsOneWidget);
    expect(iconFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder[0], 's');
    await tester.pump();
    Finder wrongLetterFinder = find.text('s');
    expect(wrongLetterFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder[0], _randomIndicative[0]);
    await tester.pump();
    expect(nextPersonKeyFinder, findsOneWidget);
    Finder secondPersonExpectedFinder = find.text(_randomIndicative[1]);
    expect(secondPersonExpectedFinder, findsNothing);
    await tester.tap(nextPersonKeyFinder);
    await tester.pump();
    expect(secondPersonExpectedFinder, findsOneWidget);
    Finder toTestButton = find.byKey(const Key('write_forward_button'));
    for (int i = 1; i < 5; i++) {
      await tester.enterText(actualVerbFinder[i], _randomIndicative[i]);
      await tester.pump();
      expect(nextPersonKeyFinder, findsOneWidget);
      await tester.tap(nextPersonKeyFinder);
      await tester.pump();
    }
    expect(toTestButton, findsOneWidget);
  });

  testWidgets('Test the state of the verb singular only',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(true, false);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicativeList, filteredPronounList,
                'infinitive', 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.writtenMode);
          })
        ],
        child: VerbIndicativeStudyWriteModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs,
              NotificationService()),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    expect(expectedVerbFinder, findsOneWidget);
    Finder expectedVerbValueFinder = find.text('aime');
    expect(expectedVerbValueFinder, findsOneWidget);
    expect(actualVerbFinder[0], findsOneWidget);
    expect(statusIconFinder, findsOneWidget);
    expect(iconFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder[0], 's');
    await tester.pump();
    Finder wrongLetterFinder = find.text('s');
    expect(wrongLetterFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder[0], filteredIndicativeList[0]);
    await tester.pump();
    expect(nextPersonKeyFinder, findsOneWidget);
    Finder secondPersonExpectedFinder = find.text(filteredIndicativeList[1]);
    expect(secondPersonExpectedFinder, findsNothing);
    await tester.tap(nextPersonKeyFinder);
    await tester.pump();
    expect(secondPersonExpectedFinder, findsOneWidget);
    Finder toTestButton = find.byKey(const Key('write_forward_button'));
    for (int i = 1; i < 3; i++) {
      await tester.enterText(actualVerbFinder[i], filteredIndicativeList[i]);
      await tester.pump();
      expect(nextPersonKeyFinder, findsOneWidget);
      await tester.tap(nextPersonKeyFinder);
      await tester.pump();
    }
    expect(toTestButton, findsOneWidget);
  });

  testWidgets('Test the state of the verb plural only',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(false, true);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicativeList, filteredPronounList,
                'infinitive', 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.writtenMode);
          })
        ],
        child: VerbIndicativeStudyWriteModeRoute(
          SettingsController(SettingsService(prefs), ThemeMode.dark, prefs,
              NotificationService()),
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    expect(expectedVerbFinder, findsOneWidget);
    Finder expectedVerbValueFinder = find.text('aimons');
    expect(expectedVerbValueFinder, findsOneWidget);
    expect(actualVerbFinder[0], findsOneWidget);
    expect(statusIconFinder, findsOneWidget);
    expect(iconFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder[0], 's');
    await tester.pump();
    Finder wrongLetterFinder = find.text('s');
    expect(wrongLetterFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder[0], filteredIndicativeList[0]);
    await tester.pump();
    expect(nextPersonKeyFinder, findsOneWidget);
    Finder secondPersonExpectedFinder = find.text(filteredIndicativeList[1]);
    expect(secondPersonExpectedFinder, findsNothing);
    await tester.tap(nextPersonKeyFinder);
    await tester.pump();
    expect(secondPersonExpectedFinder, findsOneWidget);
    Finder toTestButton = find.byKey(const Key('write_forward_button'));
    for (int i = 1; i < 3; i++) {
      await tester.enterText(actualVerbFinder[i], filteredIndicativeList[i]);
      await tester.pump();
      expect(nextPersonKeyFinder, findsOneWidget);
      await tester.tap(nextPersonKeyFinder);
      await tester.pump();
    }
    expect(toTestButton, findsOneWidget);
  });
}

void testIconColor(WidgetTester tester, Finder iconFinder, Color color) {
  expect((tester.firstWidget(iconFinder) as Icon).color, color);
}
